@@ -9,14 +9,14 @@
 #ifndef _KWALLETD_H_
 #define _KWALLETD_H_
 
-#include <QString>
-#include <QHash>
 #include "kwalletbackend.h"
-#include <QPointer>
-#include <time.h>
-#include <stdlib.h>
-#include <QtDBus>
 #include <QDBusServiceWatcher>
+#include <QHash>
+#include <QPointer>
+#include <QString>
+#include <QtDBus>
+#include <stdlib.h>
+#include <time.h>
 
 #include "ktimeout.h"
 #include "kwalletsessionstore.h"
@@ -27,6 +27,7 @@ class KTimeout;
 // @Private
 class KWalletTransaction;
 class KWalletSessionStore;
+class KWalletFreedesktopService;
 
 class KWalletD : public QObject, protected QDBusContext
 {
@@ -36,7 +37,17 @@ public:
     KWalletD();
     ~KWalletD() override;
 
+    static QString encodeWalletName(const QString &name);
+    static QString decodeWalletName(const QString &mangledName);
+
+    int nextTransactionId() const;
+    int
+    openAsync(const QString &wallet, qlonglong wId, const QString &appid, bool handleSession, const QDBusConnection &connection, const QDBusMessage &message);
+    // Close and lock the wallet
+    // Accepts "message" for working from other QDBusContexts
+    int close(int handle, bool force, const QString &appid, const QDBusMessage &message);
 public Q_SLOTS:
+
     // Is the wallet enabled?  If not, all open() calls fail.
     bool isEnabled() const;
 
@@ -47,12 +58,10 @@ public Q_SLOTS:
     int openPath(const QString &path, qlonglong wId, const QString &appid);
 
     // Open the wallet asynchronously
-    int openAsync(const QString &wallet, qlonglong wId, const QString &appid,
-                  bool handleSession);
+    int openAsync(const QString &wallet, qlonglong wId, const QString &appid, bool handleSession);
 
     // Open and unlock the wallet with this path asynchronously
-    int openPathAsync(const QString &path, qlonglong wId, const QString &appid,
-                      bool handleSession);
+    int openPathAsync(const QString &path, qlonglong wId, const QString &appid, bool handleSession);
 
     // Close and lock the wallet
     // If force = true, will close it for all users.  Behave.  This
@@ -123,6 +132,8 @@ public Q_SLOTS:
 
     // Rename an entry.  rc=0 on success.
     int renameEntry(int handle, const QString &folder, const QString &oldName, const QString &newName, const QString &appid);
+    // Rename the wallet
+    int renameWallet(const QString &oldName, const QString &newName);
 
     // Write an entry.  rc=0 on success.
     int writeEntry(int handle, const QString &folder, const QString &key, const QByteArray &value, int entryType, const QString &appid);
@@ -167,33 +178,39 @@ Q_SIGNALS:
     void walletCreated(const QString &wallet);
     void walletOpened(const QString &wallet);
     void walletDeleted(const QString &wallet);
-    void walletClosed(const QString &wallet);
-    void walletClosed(int handle);
+    void walletClosed(const QString &wallet); // clazy:exclude=overloaded-signal
+
+    // TODO KF6 remove this signal, replaced by walletClosedId(int)
+    void walletClosed(int handle); // clazy:exclude=overloaded-signal
+
+    // since 5.81
+    void walletClosedId(int handle);
+
     void allWalletsClosed();
     void folderListUpdated(const QString &wallet);
     void folderUpdated(const QString &, const QString &);
+    void entryUpdated(const QString &, const QString &, const QString &);
+    void entryRenamed(const QString &, const QString &, const QString &, const QString &);
+    void entryDeleted(const QString &, const QString &, const QString &);
     void applicationDisconnected(const QString &wallet, const QString &application);
 
 private Q_SLOTS:
-    void slotServiceOwnerChanged(const QString &name, const QString &oldOwner,
-                                 const QString &newOwner);
+    void slotServiceOwnerChanged(const QString &name, const QString &oldOwner, const QString &newOwner);
     void emitWalletListDirty();
     void timedOutClose(int handle);
     void timedOutSync(int handle);
     void notifyFailures();
     void processTransactions();
     void activatePasswordDialog();
-    void registerKWalletd4Service();
 #ifdef Q_WS_X11
     void connectToScreenSaver();
 #endif
 
 private:
     // Internal - open a wallet
-    int internalOpen(const QString &appid, const QString &wallet, bool isPath, WId w,
-                     bool modal, const QString &service);
+    int internalOpen(const QString &appid, const QString &wallet, bool isPath, WId w, bool modal, const QString &service);
     // Internal - close this wallet.
-    int internalClose(KWallet::Backend * const w, const int handle, const bool force, const bool saveBeforeClose = true);
+    int internalClose(KWallet::Backend *const w, const int handle, const bool force, const bool saveBeforeClose = true);
 
     bool isAuthorizedApp(const QString &appid, const QString &wallet, WId w);
     // This also validates the handle.  May return NULL.
@@ -203,15 +220,16 @@ private:
     // Emit signals about closing wallets
     void doCloseSignals(int, const QString &);
     void emitFolderUpdated(const QString &, const QString &);
+    void emitEntryUpdated(const QString &, const QString &, const QString &);
+    void emitEntryRenamed(const QString &, const QString &, const QString &, const QString &);
+    void emitEntryDeleted(const QString &, const QString &, const QString &);
     // Implicitly allow access for this application
     bool implicitAllow(const QString &wallet, const QString &app);
     bool implicitDeny(const QString &wallet, const QString &app);
 
     void doTransactionChangePassword(const QString &appid, const QString &wallet, qlonglong wId);
-    void doTransactionOpenCancelled(const QString &appid, const QString &wallet,
-                                    const QString &service);
-    int doTransactionOpen(const QString &appid, const QString &wallet, bool isPath,
-                          qlonglong wId, bool modal, const QString &service);
+    void doTransactionOpenCancelled(const QString &appid, const QString &wallet, const QString &service);
+    int doTransactionOpen(const QString &appid, const QString &wallet, bool isPath, qlonglong wId, bool modal, const QString &service);
     void initiateSync(int handle);
 
     void setupDialog(QWidget *dialog, WId wId, const QString &appid, bool modal);
@@ -236,7 +254,7 @@ private:
 
     KWalletTransaction *_curtrans; // current transaction
     QList<KWalletTransaction *> _transactions;
-    QPointer< QWidget > activeDialog;
+    QPointer<QWidget> activeDialog;
 
 #ifdef Q_WS_X11
     QDBusInterface *screensaver;
@@ -246,6 +264,8 @@ private:
     KWalletSessionStore _sessions;
     QDBusServiceWatcher _serviceWatcher;
 
+    std::unique_ptr<KWalletFreedesktopService> _fdoService;
+
     bool _useGpg;
 };
 
